use patina::platform::render as pr;
use patina::geom::*;

pub trait ToSDL {
    type SDLEquivalent;
    fn to_sdl(self) -> Self::SDLEquivalent;
}

impl ToSDL for pr::Colour {
    type SDLEquivalent = sdl2::pixels::Color;
    fn to_sdl(self) -> Self::SDLEquivalent {
        sdl2::pixels::Color::RGBA(self.r(), self.g(), self.b(), self.a())
    }
}

impl ToSDL for IRect {
    type SDLEquivalent = sdl2::rect::Rect;
    fn to_sdl(self) -> Self::SDLEquivalent {
        sdl2::rect::Rect::new(
            self.base().x,
            self.base().y,
            self.size().x as u32,
            self.size().y as u32,
        )
    }
}

pub trait FromSDL {
    type PatinaEquivalent;
    fn from_sdl(self) -> Self::PatinaEquivalent;
}

impl FromSDL for sdl2::rect::Rect {
    type PatinaEquivalent = IRect;
    fn from_sdl(self) -> Self::PatinaEquivalent {
        IRect::new_from_size(IPoint { x: self.x, y: self.y }, IVector { x: self.w, y: self.h })
    }
}
