// use patina::prelude::*;

use patina::{
    layout::SizePolicy,
    widget::Widget,
};

struct SimpleRoot {
    frame: patina::widget::Frame<Self>,
}

#[derive(Debug)]
enum SimpleRootMsg {
    CloseRequest,
}

impl patina::component::Component for SimpleRoot {
    type ParentMsg = patina::UIControlMsg;
    type Msg = SimpleRootMsg;

    fn process(&mut self, msg: Self::Msg) -> Vec<Self::ParentMsg> {
        match msg {
            SimpleRootMsg::CloseRequest => vec![patina::UIControlMsg::Terminate],
        }
    }
}

impl patina::UIComponent for SimpleRoot {
    fn build(bc: &patina::widget::BuildContext) -> Self {
        let mut frame = patina::widget::Frame::<Self>::new(bc);
        frame
            .layout_node_mut()
            .set_width_policy(SizePolicy {
                minimum: 100,
                desired: 200,
                slack_weight: 1,
            })
            .set_height_policy(SizePolicy {
                minimum: 100,
                desired: 200,
                slack_weight: 1,
            })
            .set_label("frame");

        let mut label = patina::widget::Label::<Self>::new(bc);
        label.layout_node_mut().set_height_policy(SizePolicy { minimum: 20, desired: 20, slack_weight: 0 }).set_width_policy(SizePolicy { minimum: 10, desired: 100, slack_weight: 1 }).set_label("label");
        frame.set_child(Box::new(label));

        Self { frame }
    }

    fn map_ui_event(&self, uievent: patina::platform::event::UIEvent) -> Option<Self::Msg> {
        match uievent {
            patina::platform::event::UIEvent::CloseRequest => Some(SimpleRootMsg::CloseRequest),
            _ => None,
        }
    }
    type RootWidget = patina::widget::Frame<Self>;
    fn root_widget(&self) -> &Self::RootWidget {
        &self.frame
    }
    fn root_widget_mut(&mut self) -> &mut Self::RootWidget {
        &mut self.frame
    }
}

fn main() {
    let ui = patina::UI::<_, SimpleRoot>::new(patina_sdl::init_sdl_platform(
        patina_sdl::SDLInitFlags::default(),
    ));
    ui.run();
}
