use crate::{
    layout::{LayoutChildIter, LayoutNode, LayoutNodeAccess, LayoutNodeContainer}, platform::{render::Painter, TextInterface}, Component, Widget
};

pub struct PlainGroup<C: Component> {
    lnode: LayoutNode,
    children: Vec<Box<dyn Widget<C>>>,
}

impl<C: Component> std::ops::Deref for PlainGroup<C> {
    type Target = LayoutNode;
    fn deref(&self) -> &Self::Target {
        &self.lnode
    }
}

impl<C: Component> LayoutNodeContainer for PlainGroup<C> {
    fn layout_node(&self) -> &LayoutNode {
        &self.lnode
    }

    fn layout_child(&self, ndx: usize) -> Option<LayoutNodeAccess> {
        self.children.get(ndx).map(|w| w.layout_node())
    }

    fn layout_child_count(&self) -> usize {
        self.children.len()
    }
}

impl<C: Component> Widget<C> for PlainGroup<C> {
    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(self)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        &mut self.lnode
    }
    fn poll(
        &mut self,
        input_state: Option<&crate::input::InputState>,
    ) -> Vec<<C as Component>::Msg> {
        vec![]
    }
    fn render(&self, painter: &mut dyn Painter, ti: &dyn TextInterface) {
        for child in self.children.iter() {
            child.render(painter, ti);
        }
    }
}
