use crate::{layout::{LayoutNode, LayoutNodeAccess, LeafLayoutNode}, platform::{render::{Colour, Painter}, TextInterface}, Component, Widget};

use super::BuildContext;

pub struct Label<C: Component> {
    lnode: LeafLayoutNode,
    _ghost: std::marker::PhantomData<C>,
}

impl<C: Component> Label<C> {
    pub fn new(bc: &BuildContext) -> Self {
        Self {
            lnode: LayoutNode::new(bc.lcache.clone()).into(),
            _ghost: Default::default(),
        }
    }
}

impl<C: Component> Widget<C> for Label<C> {
    fn poll(&mut self, input_state: Option<&crate::input::InputState>) -> Vec<<C as Component>::Msg> {
        vec![]
    }

    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(&self.lnode)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        &mut self.lnode
    }

    fn render(&self, painter: &mut dyn Painter, _ti: &dyn TextInterface) {
        painter.fill(self.layout_node().cached_rect().unwrap(), Colour::new_rgb(64, 64, 64));
    }
}
